==> indexvec-0.1.3.crate: indexvec-0.1.3/src/lib.rs <==
#![feature(unboxed_closures)]
#![feature(fn_traits)]

mod idx;
mod idx_vec;

pub use idx::Idx;
pub use idx_vec::{Enumerated, IndexVec, IntoIdx};

#[cfg(feature = "serde")]
pub use serde::{Deserialize, Serialize};

#[cfg(feature = "serde")]
fn _is_serde<T: Serialize + serde::de::DeserializeOwned>(_: &T) {
}

#[cfg(feature = "serde")]
fn _idx_is_serde() {
    newtype_index!(Foo);
    _is_serde(&Foo::new(0));
}

#[macro_export]
macro_rules! newtype_index {
    (
        $(#[$attrs:meta])*
        $vis:vis struct $type:ident {
            $( $const_vis:vis const $constant:ident = $value:expr; )*
        }
    ) => {
        $(#[$attrs])*
        #[derive(Debug, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
        #[cfg_attr(feature = "serde", derive($crate::Serialize, $crate::Deserialize))]
        $vis struct $type {
            idx: usize,
        }

        impl $type {
            $( $const_vis const $constant: $type = Self::const_new($value); )*
        }

        impl $type {
            #[allow(unused)]
            pub const fn const_new(idx: usize) -> Self {
                Self { idx }
            }
        }

        impl Clone for $type {
            #[inline]
            fn clone(&self) -> Self {
                *self
            }
        }

        impl $crate::Idx for $type {
            #[inline]
            fn new(idx: usize) -> Self {
                Self { idx }
            }

            #[inline]
            fn index(self) -> usize {
                self.idx
            }
        }
    };
    ($vis:vis $type:ident) => {
        $crate::newtype_index!($vis struct $type {});
    };
}

#[cfg(test)]
mod tests;

==> indexvec-0.2.0.crate: indexvec-0.2.0/src/lib.rs <==
#![feature(unboxed_closures)]
#![feature(fn_traits)]

mod idx;
mod idx_vec;

pub use idx::Idx;
pub use idx_vec::{Enumerated, IndexVec, IntoIdx};

// Serde is not a feature for now as `#[cfg_attr(feature = "serde", derive(..)]`
// doesn't seem to work as intended.
pub use serde::{Deserialize, Serialize};

#[macro_export]
macro_rules! newtype_index {
    (
        $(#[$attrs:meta])*
        $vis:vis struct $type:ident {
            $( $const_vis:vis const $constant:ident = $value:expr; )*
        }
    ) => {
        $(#[$attrs])*
        #[derive(Debug, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
        #[derive($crate::Serialize, $crate::Deserialize)]
        $vis struct $type {
            idx: usize,
        }

        impl $type {
            $( $const_vis const $constant: $type = Self::const_new($value); )*
        }

        impl $type {
            #[allow(unused)]
            pub const fn const_new(idx: usize) -> Self {
                Self { idx }
            }
        }

        impl Clone for $type {
            #[inline]
            fn clone(&self) -> Self {
                *self
            }
        }

        impl $crate::Idx for $type {
            #[inline]
            fn new(idx: usize) -> Self {
                Self { idx }
            }

            #[inline]
            fn index(self) -> usize {
                self.idx
            }
        }
    };
    ($vis:vis $type:ident) => {
        $crate::newtype_index!($vis struct $type {});
    };
}

#[cfg(test)]
mod tests;
